@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: gen.c,v 1.65.2.7.18.2 2006/10/04 06:50:49 marka Exp $ */
+/* $Id: gen.c,v 1.65.2.5.2.6.18.2 2006/10/04 07:06:02 marka Exp $ */
 
 #include <config.h>
 
@@ -41,7 +41,7 @@
 #include "gen-unix.h"
 #endif
 
-#define FROMTEXTARGS "rdclass, type, lexer, origin, downcase, target, callbacks"
+#define FROMTEXTARGS "rdclass, type, lexer, origin, options, target, callbacks"
 #define FROMTEXTCLASS "rdclass"
 #define FROMTEXTTYPE "type"
 #define FROMTEXTDEF "result = DNS_R_UNKNOWN"
@@ -51,7 +51,7 @@
 #define TOTEXTTYPE "rdata->type"
 #define TOTEXTDEF "use_default = ISC_TRUE"
 
-#define FROMWIREARGS "rdclass, type, source, dctx, downcase, target"
+#define FROMWIREARGS "rdclass, type, source, dctx, options, target"
 #define FROMWIRECLASS "rdclass"
 #define FROMWIRETYPE "type"
 #define FROMWIREDEF "use_default = ISC_TRUE"
@@ -91,10 +91,20 @@
 #define DIGESTTYPE "rdata->type"
 #define DIGESTDEF "use_default = ISC_TRUE"
 
+#define CHECKOWNERARGS "name, rdclass, type, wildcard"
+#define CHECKOWNERCLASS "rdclass"
+#define CHECKOWNERTYPE "type"
+#define CHECKOWNERDEF "result = ISC_TRUE"
+
+#define CHECKNAMESARGS "rdata, owner, bad"
+#define CHECKNAMESCLASS "rdata->rdclass"
+#define CHECKNAMESTYPE "rdata->type"
+#define CHECKNAMESDEF "result = ISC_TRUE"
+
 const char copyright[] =
 "/*\n"
-" * Copyright (C) 2004%s  Internet Systems Consortium, Inc. (\"ISC\")\n"
-" * Copyright (C) 1998-2003  Internet Software Consortium.\n"
+" * Copyright (C) 2004%s Internet Systems Consortium, Inc. (\"ISC\")\n"
+" * Copyright (C) 1998-2003 Internet Software Consortium.\n"
 " *\n"
 " * Permission to use, copy, modify, and distribute this software for any\n"
 " * purpose with or without fee is hereby granted, provided that the above\n"
@@ -209,7 +219,7 @@ doswitch(const char *name, const char *function, const char *args,
 	if (res == NULL)
 		result = "";
 
-	for (tt = types; tt != NULL ; tt = tt->next) {
+	for (tt = types; tt != NULL; tt = tt->next) {
 		if (first) {
 			fprintf(stdout, "\n#define %s \\\n", name);
 			fprintf(stdout, "\tswitch (%s) { \\\n" /*}*/, tsw);
@@ -272,7 +282,7 @@ dodecl(char *type, char *function, char *args) {
 	char buf1[11], buf2[11];
 
 	fputs("\n", stdout);
-	for (tt = types; tt ; tt = tt->next)
+	for (tt = types; tt; tt = tt->next)
 		if (tt->rdclass)
 			fprintf(stdout,
 				"static inline %s %s_%s_%s(%s);\n",
@@ -363,7 +373,7 @@ void
 add(int rdclass, const char *classname, int type, const char *typename,
     const char *dirname)
 {
-	struct tt *newtt = (struct tt *)malloc(sizeof *newtt);
+	struct tt *newtt = (struct tt *)malloc(sizeof(*newtt));
 	struct tt *tt, *oldtt;
 	struct cc *newcc;
 	struct cc *cc, *oldcc;
@@ -412,7 +422,7 @@ add(int rdclass, const char *classname, int type, const char *typename,
 	if (rdclass == 0)
 		return;
 
-	newcc = (struct cc *)malloc(sizeof *newcc);
+	newcc = (struct cc *)malloc(sizeof(*newcc));
 	newcc->rdclass = rdclass;
 	strcpy(newcc->classname, classname);
 	cc = classes;
@@ -437,8 +447,8 @@ add(int rdclass, const char *classname, int type, const char *typename,
 
 void
 sd(int rdclass, const char *classname, const char *dirname, char filetype) {
-	char buf[sizeof "0123456789_65535.h"];
-	char fmt[sizeof "%10[-0-9a-z]_%d.h"];
+	char buf[sizeof("0123456789_65535.h")];
+	char fmt[sizeof("%10[-0-9a-z]_%d.h")];
 	int type;
 	char typename[11];
 	isc_dir_t dir;
@@ -506,7 +516,7 @@ main(int argc, char **argv) {
 	char *file = NULL;
 	isc_dir_t dir;
 
-	for (i = 0; i < TYPENAMES ; i++)
+	for (i = 0; i < TYPENAMES; i++)
 		memset(&typenames[i], 0, sizeof(typenames[i]));
 
 	strcpy(srcdir, "");
@@ -582,7 +592,7 @@ main(int argc, char **argv) {
 	sd(0, "", buf, filetype);
 
 	if (time(&now) != -1) {
-		if ((tm = localtime(&now)) != NULL && tm->tm_year > 104)
+		if ((tm = localtime(&now)) != NULL && tm->tm_year > 104) 
 			sprintf(year, "-%d", tm->tm_year + 1900);
 		else
 			year[0] = 0;
@@ -599,7 +609,7 @@ main(int argc, char **argv) {
 		fputs("#include <isc/result.h>\n\n", stdout);
 		fputs("#include <dns/name.h>\n\n", stdout);
 
-		for (tt = types; tt != NULL ; tt = tt->next)
+		for (tt = types; tt != NULL; tt = tt->next)
 			fprintf(stdout, "#include \"%s/%s_%d.c\"\n",
 				tt->dirname, tt->typename, tt->type);
 
@@ -627,6 +637,12 @@ main(int argc, char **argv) {
 		doswitch("DIGESTSWITCH", "digest",
 			 DIGESTARGS, DIGESTTYPE,
 			 DIGESTCLASS, DIGESTDEF);
+		doswitch("CHECKOWNERSWITCH", "checkowner",
+			CHECKOWNERARGS, CHECKOWNERTYPE,
+			CHECKOWNERCLASS, CHECKOWNERDEF);
+		doswitch("CHECKNAMESSWITCH", "checknames",
+			CHECKNAMESARGS, CHECKNAMESTYPE,
+			CHECKNAMESCLASS, CHECKNAMESDEF);
 
 		/*
 		 * From here down, we are processing the rdata names and
@@ -658,31 +674,6 @@ main(int argc, char **argv) {
 		insert_into_typenames(254, "maila", METAQUESTIONONLY);
 		insert_into_typenames(255, "any", METAQUESTIONONLY);
 
-		fprintf(stdout, "\ntypedef struct {\n");
-		fprintf(stdout, "\tconst char *name;\n");
-		fprintf(stdout, "\tunsigned int flags;\n");
-		fprintf(stdout, "} typeattr_t;\n");
-		fprintf(stdout, "static typeattr_t typeattr[] = {\n");
-		for (i = 0; i <= maxtype ; i++) {
-			ttn = find_typename(i);
-			if (ttn == NULL) {
-				const char *attrs;
-				if (i >= 128 && i < 255)
-					attrs = "DNS_RDATATYPEATTR_UNKNOWN | "
-						"DNS_RDATATYPEATTR_META";
-				else
-					attrs = "DNS_RDATATYPEATTR_UNKNOWN";
-				fprintf(stdout, "\t{ \"TYPE%d\", %s}%s\n",
-					i, attrs, PRINT_COMMA(i));
-			} else {
-				fprintf(stdout, "\t{ \"%s\", %s }%s\n",
-				       upper(ttn->typename),
-				       upper(ttn->attr),
-				       PRINT_COMMA(i));
-			}
-		}
-		fprintf(stdout, "};\n");
-
 		/*
 		 * Spit out a quick and dirty hash function.  Here,
 		 * we walk through the list of type names, and calculate
@@ -698,7 +689,7 @@ main(int argc, char **argv) {
 		fprintf(stdout, "\t\tif (sizeof(_s) - 1 == _n && \\\n"
 				"\t\t    strncasecmp(_s,(_tn),"
 				"(sizeof(_s) - 1)) == 0) { \\\n");
-		fprintf(stdout, "\t\t\tif ((typeattr[_d].flags & "
+		fprintf(stdout, "\t\t\tif ((dns_rdatatype_attributes(_d) & "
 		       		  "DNS_RDATATYPEATTR_RESERVED) != 0) \\\n");
 		fprintf(stdout, "\t\t\t\treturn (ISC_R_NOTIMPLEMENTED); \\\n");
 		fprintf(stdout, "\t\t\t*(_tp) = _d; \\\n");
@@ -709,7 +700,7 @@ main(int argc, char **argv) {
 		fprintf(stdout, "#define RDATATYPE_FROMTEXT_SW(_hash,"
 				"_typename,_length,_typep) \\\n");
 		fprintf(stdout, "\tswitch (_hash) { \\\n");
-		for (i = 0; i <= maxtype ; i++) {
+		for (i = 0; i <= maxtype; i++) {
 			ttn = find_typename(i);
 			if (ttn == NULL)
 				continue;
@@ -727,7 +718,7 @@ main(int argc, char **argv) {
 			 * Find all other entries that happen to match
 			 * this hash.
 			 */
-			for (j = 0; j <= maxtype ; j++) {
+			for (j = 0; j <= maxtype; j++) {
 				ttn2 = find_typename(j);
 				if (ttn2 == NULL)
 					continue;
@@ -743,6 +734,29 @@ main(int argc, char **argv) {
 		}
 		fprintf(stdout, "\t}\n");
 
+		fprintf(stdout, "#define RDATATYPE_ATTRIBUTE_SW \\\n");
+		fprintf(stdout, "\tswitch (type) { \\\n");
+		for (i = 0; i <= maxtype; i++) {
+			ttn = find_typename(i);
+			if (ttn == NULL)
+				continue;
+			fprintf(stdout, "\tcase %u: return (%s); \\\n",
+			        i, upper(ttn->attr));
+		}
+		fprintf(stdout, "\t}\n");
+
+		fprintf(stdout, "#define RDATATYPE_TOTEXT_SW \\\n");
+		fprintf(stdout, "\tswitch (type) { \\\n");
+		for (i = 0; i <= maxtype; i++) {
+			ttn = find_typename(i);
+			if (ttn == NULL)
+				continue;
+			fprintf(stdout, "\tcase %u: return "
+				"(str_totext(\"%s\", target)); \\\n",
+			        i, upper(ttn->typename));
+		}
+		fprintf(stdout, "\t}\n");
+
 		fputs("#endif /* DNS_CODE_H */\n", stdout);
 	} else if (type_enum) {
 		char *s;
@@ -754,7 +768,7 @@ main(int argc, char **argv) {
 		fprintf(stdout, "\tdns_rdatatype_none = 0,\n");
 
 		lasttype = 0;
-		for (tt = types; tt != NULL ; tt = tt->next)
+		for (tt = types; tt != NULL; tt = tt->next)
 			if (tt->type != lasttype)
 				fprintf(stdout,
 					"\tdns_rdatatype_%s = %d,\n",
@@ -772,7 +786,7 @@ main(int argc, char **argv) {
 		fprintf(stdout, "#define dns_rdatatype_none\t"
 			"((dns_rdatatype_t)dns_rdatatype_none)\n");
 
-		for (tt = types; tt != NULL ; tt = tt->next)
+		for (tt = types; tt != NULL; tt = tt->next)
 			if (tt->type != lasttype) {
 				s = funname(tt->typename, buf1);
 				fprintf(stdout,
@@ -837,29 +851,29 @@ main(int argc, char **argv) {
 	} else if (structs) {
 		if (prefix != NULL) {
 			if ((fd = fopen(prefix,"r")) != NULL) {
-				while (fgets(buf, sizeof buf, fd) != NULL)
+				while (fgets(buf, sizeof(buf), fd) != NULL)
 					fputs(buf, stdout);
 				fclose(fd);
 			}
 		}
-		for (tt = types; tt != NULL ; tt = tt->next) {
+		for (tt = types; tt != NULL; tt = tt->next) {
 			sprintf(buf, "%s/%s_%d.h",
 				tt->dirname, tt->typename, tt->type);
 			if ((fd = fopen(buf,"r")) != NULL) {
-				while (fgets(buf, sizeof buf, fd) != NULL)
+				while (fgets(buf, sizeof(buf), fd) != NULL)
 					fputs(buf, stdout);
 				fclose(fd);
 			}
 		}
 		if (suffix != NULL) {
 			if ((fd = fopen(suffix,"r")) != NULL) {
-				while (fgets(buf, sizeof buf, fd) != NULL)
+				while (fgets(buf, sizeof(buf), fd) != NULL)
 					fputs(buf, stdout);
 				fclose(fd);
 			}
 		}
 	} else if (depend) {
-		for (tt = types; tt != NULL ; tt = tt->next)
+		for (tt = types; tt != NULL; tt = tt->next)
 			fprintf(stdout, "%s:\t%s/%s_%d.h\n", file,
 				tt->dirname, tt->typename, tt->type);
 	}
